#include <stdafx.h>
#include "WriteBuffer.h"
#include "httpclient.h"

namespace httpbase
{

WriteFileBuffer::WriteFileBuffer(LPCTSTR filename)
{
	lstrcpy( m_filename, filename );
	m_open = FALSE;
}

WriteFileBuffer::~WriteFileBuffer()
{
	if( m_open )
	{
		Close();
	}
	m_filename[0] = 0;
}

BOOL WriteFileBuffer::Open()
{
	HRESULT hr = m_file.Create();
	if( FAILED(hr) )
	{
		return FALSE;
	}

	m_open = TRUE;
	return TRUE;
}

int WriteFileBuffer::Write( const BYTE *data, int size )
{
	DWORD readed;
	HRESULT hr = m_file.Write( data, size, &readed );
	if( FAILED(hr) )
	{
		return -1;
	}

	return (int)readed;
}

BOOL WriteFileBuffer::Close()
{
	HRESULT hr = m_file.Close(m_filename);
	if( FAILED(hr) )
	{
		return FALSE;
	}

	m_open = FALSE;
	return TRUE;
}

WritePreparedMemoryBuffer::WritePreparedMemoryBuffer( DWORD filesize, BYTE **output_buffer)
{
	m_buffer = output_buffer;
	m_filesize = filesize;
	m_currentSize = 0;
}

WritePreparedMemoryBuffer::~WritePreparedMemoryBuffer()
{
	if( NULL != m_internalBuffer )
	{
		::CoTaskMemFree(m_internalBuffer);
	}

	m_buffer = NULL;
	m_internalBuffer = NULL;
	m_filesize = 0;
	m_currentSize = 0;
}

BOOL WritePreparedMemoryBuffer::Open()
{
	m_internalBuffer = (BYTE *)::CoTaskMemAlloc(m_filesize+1);
	if( NULL == m_internalBuffer )
		return FALSE;

	return TRUE;
}

int WritePreparedMemoryBuffer::Write( const BYTE *data, int size )
{
	if( ( m_currentSize + size ) > (int)m_filesize )
	{
		return -1;
	}

	::CopyMemory( m_internalBuffer + m_currentSize, data, size );
	m_currentSize += size;
	return size;
}

BOOL WritePreparedMemoryBuffer::Close()
{
	*m_buffer = m_internalBuffer;
	m_internalBuffer[m_filesize] =0;
	m_internalBuffer = NULL;
	return TRUE;
}

WriteAutoRaisedMemoryBuffer::WriteAutoRaisedMemoryBuffer( BYTE **output_buffer)
{
	m_buffer = output_buffer;
	m_currentSize = 0;
	m_maxSize = 1024*16;
}

WriteAutoRaisedMemoryBuffer::~WriteAutoRaisedMemoryBuffer()
{
	if( NULL != m_internalBuffer )
	{
		::CoTaskMemFree(m_internalBuffer);
	}

	m_buffer = NULL;
	m_internalBuffer = NULL;
	m_currentSize = 0;
	m_maxSize = 0;
}

BOOL WriteAutoRaisedMemoryBuffer::Open()
{
	m_internalBuffer = (BYTE *)::CoTaskMemAlloc(m_maxSize);
	if( NULL == m_internalBuffer )
		return FALSE;

	return TRUE;
}

BOOL WriteAutoRaisedMemoryBuffer::RaiseMemory( const int wantSize )
{
	while( wantSize > m_maxSize )
	{
		m_maxSize *= 2;
	}

	BYTE *newBuffer = (BYTE *)::CoTaskMemAlloc( m_maxSize+1 );
	if( NULL == newBuffer )
	{
		return FALSE;
	}

	::CopyMemory( newBuffer, m_internalBuffer, m_currentSize );

	BYTE *delete_buffer = m_internalBuffer;	
	m_internalBuffer = newBuffer;

	::CoTaskMemFree(delete_buffer);
	return TRUE;
}

int WriteAutoRaisedMemoryBuffer::Write( const BYTE *data, int size )
{
	if( ( m_currentSize + size ) > m_maxSize )
	{
		if( FALSE == RaiseMemory( m_currentSize + size ) )
			return -1;
	}

	::CopyMemory( m_internalBuffer + m_currentSize, data, size );
	m_currentSize += size;
	return size;
}

BOOL WriteAutoRaisedMemoryBuffer::Close()
{
	*m_buffer = m_internalBuffer;
	m_internalBuffer[m_currentSize] = 0;
	m_internalBuffer = NULL;
	return TRUE;
}

};